@@ -1,4 +1,4 @@
-<?php
+<?php declare(strict_types=1);
 /*
  * This file is part of PHPLOC.
  *
@@ -9,10 +9,61 @@
  */
 namespace SebastianBergmann\PHPLOC;
 
-/**
- * PHPLOC code analyser.
- */
-class Analyser
+use const T_ABSTRACT;
+use const T_BOOLEAN_AND;
+use const T_BOOLEAN_OR;
+use const T_CASE;
+use const T_CATCH;
+use const T_CLASS;
+use const T_COMMENT;
+use const T_CONST;
+use const T_CONSTANT_ENCAPSED_STRING;
+use const T_CURLY_OPEN;
+use const T_DECLARE;
+use const T_DOC_COMMENT;
+use const T_DOLLAR_OPEN_CURLY_BRACES;
+use const T_DOUBLE_COLON;
+use const T_ELSEIF;
+use const T_EXTENDS;
+use const T_FINAL;
+use const T_FOR;
+use const T_FOREACH;
+use const T_FUNCTION;
+use const T_GLOBAL;
+use const T_IF;
+use const T_INTERFACE;
+use const T_LOGICAL_AND;
+use const T_LOGICAL_OR;
+use const T_NAMESPACE;
+use const T_NEW;
+use const T_NS_SEPARATOR;
+use const T_OBJECT_OPERATOR;
+use const T_PRIVATE;
+use const T_PROTECTED;
+use const T_PUBLIC;
+use const T_STATIC;
+use const T_STRING;
+use const T_TRAIT;
+use const T_USE;
+use const T_VARIABLE;
+use const T_WHILE;
+use const T_WHITESPACE;
+use function array_pop;
+use function count;
+use function file_get_contents;
+use function in_array;
+use function is_array;
+use function is_string;
+use function rtrim;
+use function str_replace;
+use function strpos;
+use function strtolower;
+use function substr;
+use function substr_count;
+use function token_get_all;
+use function trim;
+
+final class Analyser
 {
     /**
      * @var Collector
@@ -45,17 +96,10 @@ class Analyser
 
     public function __construct()
     {
-        $this->collector = new Collector();
+        $this->collector = new Collector;
     }
 
-    /**
-     * Processes a set of files.
-     *
-     * @param bool $countTests
-     *
-     * @return array
-     */
-    public function countFiles(array $files, $countTests)
+    public function countFiles(array $files, bool $countTests)
     {
         foreach ($files as $file) {
             $this->countFile($file, $countTests);
@@ -64,37 +108,32 @@ class Analyser
         return $this->collector->getPublisher()->toArray();
     }
 
-    /**
-     * Pre-processes a single file.
-     *
-     * @param string $filename
-     */
-    public function preProcessFile($filename): void
+    public function preProcessFile(string $filename): void
     {
-        $tokens    = \token_get_all(\file_get_contents($filename));
-        $numTokens = \count($tokens);
+        $tokens    = token_get_all(file_get_contents($filename));
+        $numTokens = count($tokens);
         $namespace = false;
 
         for ($i = 0; $i < $numTokens; $i++) {
-            if (\is_string($tokens[$i])) {
+            if (is_string($tokens[$i])) {
                 continue;
             }
 
             switch ($tokens[$i][0]) {
-                case \T_NAMESPACE:
+                case T_NAMESPACE:
                     $namespace = $this->getNamespaceName($tokens, $i);
 
                     break;
 
-                case \T_CLASS:
+                case T_CLASS:
                     if (!$this->isClassDeclaration($tokens, $i)) {
                         break;
                     }
 
                     $className = $this->getClassName($namespace, $tokens, $i);
 
-                    if (isset($tokens[$i + 4]) && \is_array($tokens[$i + 4]) &&
-                        $tokens[$i + 4][0] == \T_EXTENDS) {
+                    if (isset($tokens[$i + 4]) && is_array($tokens[$i + 4]) &&
+                        $tokens[$i + 4][0] === T_EXTENDS) {
                         $parent = $this->getClassName($namespace, $tokens, $i + 4);
                     } else {
                         $parent = null;
@@ -119,73 +158,78 @@ class Analyser
             $this->preProcessFile($filename);
         }
 
-        $buffer              = \file_get_contents($filename);
-        $this->collector->incrementLines(\substr_count($buffer, "\n"));
-        $tokens              = \token_get_all($buffer);
-        $numTokens           = \count($tokens);
+        $buffer = file_get_contents($filename);
+        $this->collector->incrementLines(substr_count($buffer, "\n"));
+        $tokens    = token_get_all($buffer);
+        $numTokens = count($tokens);
 
         unset($buffer);
 
         $this->collector->addFile($filename);
 
-        $blocks            = [];
-        $currentBlock      = false;
-        $namespace         = false;
-        $className         = null;
-        $functionName      = null;
-        $testClass         = false;
+        $blocks       = [];
+        $currentBlock = false;
+        $namespace    = false;
+        $className    = null;
+        $functionName = null;
+        $testClass    = false;
         $this->collector->currentClassReset();
-        $isInMethod = false;
+        $isLogicalLine = true;
+        $isInMethod    = false;
 
         for ($i = 0; $i < $numTokens; $i++) {
-            if (\is_string($tokens[$i])) {
-                $token = \trim($tokens[$i]);
+            if (is_string($tokens[$i])) {
+                $token = trim($tokens[$i]);
 
-                if ($token == ';') {
-                    if ($className !== null && !$testClass) {
-                        $this->collector->currentClassIncrementLines();
+                if ($token === ';') {
+                    if ($isLogicalLine) {
+                        if ($className !== null && !$testClass) {
+                            $this->collector->currentClassIncrementLines();
 
-                        if ($functionName !== null) {
-                            $this->collector->currentMethodIncrementLines();
+                            if ($functionName !== null) {
+                                $this->collector->currentMethodIncrementLines();
+                            }
+                        } elseif ($functionName !== null) {
+                            $this->collector->incrementFunctionLines();
                         }
-                    } elseif ($functionName !== null) {
-                        $this->collector->incrementFunctionLines();
-                    }
 
-                    $this->collector->incrementLogicalLines();
-                } elseif ($token == '?' && !$testClass) {
+                        $this->collector->incrementLogicalLines();
+                    }
+                    $isLogicalLine = true;
+                } elseif ($token === '?' && !$testClass) {
                     if ($className !== null) {
                         $this->collector->currentClassIncrementComplexity();
                         $this->collector->currentMethodIncrementComplexity();
                     }
 
                     $this->collector->incrementComplexity();
-                } elseif ($token == '{') {
-                    if ($currentBlock == \T_CLASS) {
+                } elseif ($token === '{') {
+                    if ($currentBlock == T_CLASS) {
                         $block = $className;
-                    } elseif ($currentBlock == \T_FUNCTION) {
+                    } elseif ($currentBlock == T_FUNCTION) {
                         $block = $functionName;
                     } else {
                         $block = false;
                     }
 
-                    \array_push($blocks, $block);
+                    $blocks[] = $block;
 
                     $currentBlock = false;
-                } elseif ($token == '}') {
-                    $block = \array_pop($blocks);
+                } elseif ($token === '}') {
+                    $block = array_pop($blocks);
 
                     if ($block !== false && $block !== null) {
-                        if ($block == $functionName) {
+                        if ($block === $functionName) {
                             $functionName = null;
 
                             if ($isInMethod) {
                                 $this->collector->currentMethodStop();
                                 $isInMethod = false;
                             }
-                        } elseif ($block == $className) {
-                            $className         = null;
-                            $testClass         = false;
+                        } elseif ($block === $className) {
+                            $className = null;
+                            $testClass = false;
+                            $this->collector->currentClassStop();
                             $this->collector->currentClassReset();
                         }
                     }
@@ -197,62 +241,70 @@ class Analyser
             [$token, $value] = $tokens[$i];
 
             switch ($token) {
-                case \T_NAMESPACE:
+                case T_NAMESPACE:
                     $namespace = $this->getNamespaceName($tokens, $i);
                     $this->collector->addNamespace($namespace);
+                    $isLogicalLine = false;
 
                     break;
 
-                case \T_CLASS:
-                case \T_INTERFACE:
-                case \T_TRAIT:
+                case T_CLASS:
+                case T_INTERFACE:
+                case T_TRAIT:
                     if (!$this->isClassDeclaration($tokens, $i)) {
                         break;
                     }
 
                     $this->collector->currentClassReset();
                     $this->collector->currentClassIncrementComplexity();
-                    $className        = $this->getClassName($namespace, $tokens, $i);
-                    $currentBlock     = \T_CLASS;
+                    $className    = $this->getClassName($namespace, $tokens, $i);
+                    $currentBlock = T_CLASS;
 
-                    if ($token == \T_TRAIT) {
+                    if ($token === T_TRAIT) {
                         $this->collector->incrementTraits();
-                    } elseif ($token == \T_INTERFACE) {
+                    } elseif ($token === T_INTERFACE) {
                         $this->collector->incrementInterfaces();
                     } else {
                         if ($countTests && $this->isTestClass($className)) {
                             $testClass = true;
                             $this->collector->incrementTestClasses();
                         } else {
-                            if (isset($tokens[$i - 2]) &&
-                                \is_array($tokens[$i - 2]) &&
-                                $tokens[$i - 2][0] == \T_ABSTRACT) {
+                            $classModifierToken = $this->getPreviousNonWhitespaceNonCommentTokenPos($tokens, $i);
+
+                            if ($classModifierToken !== false &&
+                                $tokens[$classModifierToken][0] === T_ABSTRACT
+                            ) {
                                 $this->collector->incrementAbstractClasses();
+                            } elseif (
+                                $classModifierToken !== false &&
+                                $tokens[$classModifierToken][0] === T_FINAL
+                            ) {
+                                $this->collector->incrementFinalClasses();
                             } else {
-                                $this->collector->incrementConcreteClasses();
+                                $this->collector->incrementNonFinalClasses();
                             }
                         }
                     }
 
                     break;
 
-                case \T_FUNCTION:
+                case T_FUNCTION:
                     $prev = $this->getPreviousNonWhitespaceTokenPos($tokens, $i);
 
-                    if ($tokens[$prev][0] === \T_USE) {
+                    if ($tokens[$prev][0] === T_USE) {
                         break;
                     }
 
-                    $currentBlock = \T_FUNCTION;
+                    $currentBlock = T_FUNCTION;
 
                     $next = $this->getNextNonWhitespaceTokenPos($tokens, $i);
 
-                    if (!\is_array($tokens[$next]) && $tokens[$next] == '&') {
+                    if (!is_array($tokens[$next]) && $tokens[$next] === '&') {
                         $next = $this->getNextNonWhitespaceTokenPos($tokens, $next);
                     }
 
-                    if (\is_array($tokens[$next]) &&
-                        $tokens[$next][0] == \T_STRING) {
+                    if (is_array($tokens[$next]) &&
+                        $tokens[$next][0] === T_STRING) {
                         $functionName = $tokens[$next][1];
                     } else {
                         $currentBlock = 'anonymous function';
@@ -260,19 +312,19 @@ class Analyser
                         $this->collector->incrementAnonymousFunctions();
                     }
 
-                    if ($currentBlock == \T_FUNCTION) {
+                    if ($currentBlock === T_FUNCTION) {
                         if ($className === null &&
-                            $functionName != 'anonymous function') {
+                            $functionName !== 'anonymous function') {
                             $this->collector->incrementNamedFunctions();
                         } else {
                             $static     = false;
-                            $visibility = \T_PUBLIC;
+                            $visibility = T_PUBLIC;
 
                             for ($j = $i; $j > 0; $j--) {
-                                if (\is_string($tokens[$j])) {
-                                    if ($tokens[$j] == '{' ||
-                                        $tokens[$j] == '}' ||
-                                        $tokens[$j] == ';') {
+                                if (is_string($tokens[$j])) {
+                                    if ($tokens[$j] === '{' ||
+                                        $tokens[$j] === '}' ||
+                                        $tokens[$j] === ';') {
                                         break;
                                     }
 
@@ -281,17 +333,17 @@ class Analyser
 
                                 if (isset($tokens[$j][0])) {
                                     switch ($tokens[$j][0]) {
-                                        case \T_PRIVATE:
-                                            $visibility = \T_PRIVATE;
+                                        case T_PRIVATE:
+                                            $visibility = T_PRIVATE;
 
                                             break;
 
-                                        case \T_PROTECTED:
-                                            $visibility = \T_PROTECTED;
+                                        case T_PROTECTED:
+                                            $visibility = T_PROTECTED;
 
                                             break;
 
-                                        case \T_STATIC:
+                                        case T_STATIC:
                                             $static = true;
 
                                             break;
@@ -306,16 +358,20 @@ class Analyser
                                 $isInMethod = true;
                                 $this->collector->currentMethodStart();
 
+                                $this->collector->currentClassIncrementMethods();
+
                                 if (!$static) {
                                     $this->collector->incrementNonStaticMethods();
                                 } else {
                                     $this->collector->incrementStaticMethods();
                                 }
 
-                                if ($visibility == \T_PUBLIC) {
+                                if ($visibility === T_PUBLIC) {
                                     $this->collector->incrementPublicMethods();
-                                } else {
-                                    $this->collector->incrementNonPublicMethods();
+                                } elseif ($visibility === T_PROTECTED) {
+                                    $this->collector->incrementProtectedMethods();
+                                } elseif ($visibility === T_PRIVATE) {
+                                    $this->collector->incrementPrivateMethods();
                                 }
                             }
                         }
@@ -323,29 +379,29 @@ class Analyser
 
                     break;
 
-                case \T_CURLY_OPEN:
-                    $currentBlock = \T_CURLY_OPEN;
-                    \array_push($blocks, $currentBlock);
+                case T_CURLY_OPEN:
+                    $currentBlock = T_CURLY_OPEN;
+                    $blocks[]     = $currentBlock;
 
                     break;
 
-                case \T_DOLLAR_OPEN_CURLY_BRACES:
-                    $currentBlock = \T_DOLLAR_OPEN_CURLY_BRACES;
-                    \array_push($blocks, $currentBlock);
+                case T_DOLLAR_OPEN_CURLY_BRACES:
+                    $currentBlock = T_DOLLAR_OPEN_CURLY_BRACES;
+                    $blocks[]     = $currentBlock;
 
                     break;
 
-                case \T_IF:
-                case \T_ELSEIF:
-                case \T_FOR:
-                case \T_FOREACH:
-                case \T_WHILE:
-                case \T_CASE:
-                case \T_CATCH:
-                case \T_BOOLEAN_AND:
-                case \T_LOGICAL_AND:
-                case \T_BOOLEAN_OR:
-                case \T_LOGICAL_OR:
+                case T_IF:
+                case T_ELSEIF:
+                case T_FOR:
+                case T_FOREACH:
+                case T_WHILE:
+                case T_CASE:
+                case T_CATCH:
+                case T_BOOLEAN_AND:
+                case T_LOGICAL_AND:
+                case T_BOOLEAN_OR:
+                case T_LOGICAL_OR:
                     if (!$testClass) {
                         if ($isInMethod) {
                             $this->collector->currentClassIncrementComplexity();
@@ -357,29 +413,37 @@ class Analyser
 
                     break;
 
-                case \T_COMMENT:
-                case \T_DOC_COMMENT:
+                case T_COMMENT:
+                case T_DOC_COMMENT:
                     // We want to count all intermediate lines before the token ends
                     // But sometimes a new token starts after a newline, we don't want to count that.
                     // That happened with /* */ and /**  */, but not with // since it'll end at the end
-                    $this->collector->incrementCommentLines(\substr_count(\rtrim($value, "\n"), "\n") + 1);
+                    $this->collector->incrementCommentLines(substr_count(rtrim($value, "\n"), "\n") + 1);
 
                     break;
-                case \T_CONST:
-                    $this->collector->incrementClassConstants();
+                case T_CONST:
+                    $possibleScopeToken = $this->getPreviousNonWhitespaceNonCommentTokenPos($tokens, $i);
+
+                    if ($possibleScopeToken !== false
+                        && in_array($tokens[$possibleScopeToken][0], [T_PRIVATE, T_PROTECTED], true)
+                    ) {
+                        $this->collector->incrementNonPublicClassConstants();
+                    } else {
+                        $this->collector->incrementPublicClassConstants();
+                    }
 
                     break;
 
-                case \T_STRING:
-                    if ($value == 'define') {
+                case T_STRING:
+                    if ($value === 'define') {
                         $this->collector->incrementGlobalConstants();
 
                         $j = $i + 1;
 
-                        while (isset($tokens[$j]) && $tokens[$j] != ';') {
-                            if (\is_array($tokens[$j]) &&
-                                $tokens[$j][0] == \T_CONSTANT_ENCAPSED_STRING) {
-                                $this->collector->addConstant(\str_replace('\'', '', $tokens[$j][1]));
+                        while (isset($tokens[$j]) && $tokens[$j] !== ';') {
+                            if (is_array($tokens[$j]) &&
+                                $tokens[$j][0] === T_CONSTANT_ENCAPSED_STRING) {
+                                $this->collector->addConstant(str_replace('\'', '', $tokens[$j][1]));
 
                                 break;
                             }
@@ -392,44 +456,50 @@ class Analyser
 
                     break;
 
-                case \T_DOUBLE_COLON:
-                case \T_OBJECT_OPERATOR:
+                case T_DOUBLE_COLON:
+                case T_OBJECT_OPERATOR:
                     $n  = $this->getNextNonWhitespaceTokenPos($tokens, $i);
                     $nn = $this->getNextNonWhitespaceTokenPos($tokens, $n);
 
                     if ($n && $nn &&
                         isset($tokens[$n][0]) &&
-                        ($tokens[$n][0] == \T_STRING ||
-                         $tokens[$n][0] == \T_VARIABLE) &&
-                        $tokens[$nn] == '(') {
-                        if ($token == \T_DOUBLE_COLON) {
+                        ($tokens[$n][0] === T_STRING ||
+                         $tokens[$n][0] === T_VARIABLE) &&
+                        $tokens[$nn] === '(') {
+                        if ($token === T_DOUBLE_COLON) {
                             $this->collector->incrementStaticMethodCalls();
                         } else {
                             $this->collector->incrementNonStaticMethodCalls();
                         }
                     } else {
-                        if ($token == \T_DOUBLE_COLON &&
-                            $tokens[$n][0] == \T_VARIABLE) {
+                        if ($token === T_DOUBLE_COLON &&
+                            $tokens[$n][0] === T_VARIABLE) {
                             $this->collector->incrementStaticAttributeAccesses();
-                        } elseif ($token == \T_OBJECT_OPERATOR) {
+                        } elseif ($token === T_OBJECT_OPERATOR) {
                             $this->collector->incrementNonStaticAttributeAccesses();
                         }
                     }
 
                     break;
 
-                case \T_GLOBAL:
+                case T_GLOBAL:
                     $this->collector->incrementGlobalVariableAccesses();
 
                     break;
 
-                case \T_VARIABLE:
-                    if ($value == '$GLOBALS') {
+                case T_VARIABLE:
+                    if ($value === '$GLOBALS') {
                         $this->collector->incrementGlobalVariableAccesses();
                     } elseif (isset($this->superGlobals[$value])) {
                         $this->collector->incrementSuperGlobalVariableAccesses();
                     }
 
+                    break;
+
+                case T_USE:
+                case T_DECLARE:
+                    $isLogicalLine = false;
+
                     break;
             }
         }
@@ -446,7 +516,7 @@ class Analyser
             $namespace = $tokens[$i + 2][1];
 
             for ($j = $i + 3;; $j += 2) {
-                if (isset($tokens[$j]) && $tokens[$j][0] == \T_NS_SEPARATOR) {
+                if (isset($tokens[$j]) && $tokens[$j][0] === T_NS_SEPARATOR) {
                     $namespace .= '\\' . $tokens[$j + 1][1];
                 } else {
                     break;
@@ -472,11 +542,12 @@ class Analyser
         if (!isset($tokens[$i][1])) {
             return 'invalid class name';
         }
-        $className  = $tokens[$i][1];
+
+        $className = $tokens[$i][1];
 
         $namespaced = $className === '\\';
 
-        while (\is_array($tokens[$i + 1]) && $tokens[$i + 1][0] !== \T_WHITESPACE) {
+        while (isset($tokens[$i + 1]) && is_array($tokens[$i + 1]) && $tokens[$i + 1][0] !== T_WHITESPACE) {
             $className .= $tokens[++$i][1];
         }
 
@@ -484,7 +555,7 @@ class Analyser
             $className = $namespace . '\\' . $className;
         }
 
-        return \strtolower($className);
+        return strtolower($className);
     }
 
     /**
@@ -506,11 +577,11 @@ class Analyser
                 break;
             }
 
-            if ($parent == 'phpunit_framework_testcase' ||
-                $parent == '\\phpunit_framework_testcase' ||
+            if ($parent === 'phpunit_framework_testcase' ||
+                $parent === '\\phpunit_framework_testcase' ||
                 // TODO: Recognize PHPUnit\Framework\TestCase when it is imported
-                $parent == 'phpunit\\framework\\testcase' ||
-                $parent == '\\phpunit\\framework\\testcase') {
+                $parent === 'phpunit\\framework\\testcase' ||
+                $parent === '\\phpunit\\framework\\testcase') {
                 return true;
             }
 
@@ -525,7 +596,7 @@ class Analyser
 
         // Fallback: Treat the class as a test case class if the name
         // of the parent class ends with "TestCase".
-        return \substr($this->classes[$className], -8) === 'testcase';
+        return substr((string) $this->classes[$className], -8) === 'testcase';
     }
 
     /**
@@ -538,24 +609,24 @@ class Analyser
      */
     private function isTestMethod($functionName, $visibility, $static, array $tokens, $currentToken)
     {
-        if ($static || $visibility != \T_PUBLIC) {
+        if ($static || $visibility != T_PUBLIC) {
             return false;
         }
 
-        if (\strpos($functionName, 'test') === 0) {
+        if (strpos($functionName, 'test') === 0) {
             return true;
         }
 
-        while ($tokens[$currentToken][0] != \T_DOC_COMMENT) {
-            if ($tokens[$currentToken] == '{' || $tokens[$currentToken] == '}') {
+        while ($tokens[$currentToken][0] !== T_DOC_COMMENT) {
+            if ($tokens[$currentToken] === '{' || $tokens[$currentToken] === '}') {
                 return false;
             }
 
-            --$currentToken;
+            $currentToken--;
         }
 
-        return \strpos($tokens[$currentToken][1], '@test') !== false ||
-               \strpos($tokens[$currentToken][1], '@scenario') !== false;
+        return strpos($tokens[$currentToken][1], '@test') !== false ||
+               strpos($tokens[$currentToken][1], '@scenario') !== false;
     }
 
     /**
@@ -567,7 +638,7 @@ class Analyser
     {
         if (isset($tokens[$start + 1])) {
             if (isset($tokens[$start + 1][0]) &&
-                $tokens[$start + 1][0] == \T_WHITESPACE &&
+                $tokens[$start + 1][0] === T_WHITESPACE &&
                 isset($tokens[$start + 2])) {
                 return $start + 2;
             }
@@ -587,7 +658,7 @@ class Analyser
     {
         if (isset($tokens[$start - 1])) {
             if (isset($tokens[$start - 1][0]) &&
-                $tokens[$start - 1][0] == \T_WHITESPACE &&
+                $tokens[$start - 1][0] === T_WHITESPACE &&
                 isset($tokens[$start - 2])) {
                 return $start - 2;
             }
@@ -598,6 +669,29 @@ class Analyser
         return false;
     }
 
+    /**
+     * @return bool
+     */
+    private function getPreviousNonWhitespaceNonCommentTokenPos(array $tokens, int $start)
+    {
+        $previousTokenIndex = $start - 1;
+
+        if (isset($tokens[$previousTokenIndex])) {
+            if (in_array($tokens[$previousTokenIndex][0], [
+                T_WHITESPACE,
+                T_COMMENT,
+                T_DOC_COMMENT,
+            ], true)
+            ) {
+                return $this->getPreviousNonWhitespaceNonCommentTokenPos($tokens, $previousTokenIndex);
+            }
+
+            return $previousTokenIndex;
+        }
+
+        return false;
+    }
+
     /**
      * @param int $i
      *
@@ -608,7 +702,7 @@ class Analyser
         $n = $this->getPreviousNonWhitespaceTokenPos($tokens, $i);
 
         return !isset($tokens[$n])
-            || !\is_array($tokens[$n])
-            || !\in_array($tokens[$n][0], [\T_DOUBLE_COLON, \T_NEW], true);
+            || !is_array($tokens[$n])
+            || !in_array($tokens[$n][0], [T_DOUBLE_COLON, T_NEW], true);
     }
 }
