@@ -1,4 +1,4 @@
-<?php
+<?php declare(strict_types=1);
 /*
  * This file is part of PHPLOC.
  *
@@ -9,6 +9,8 @@
  */
 namespace SebastianBergmann\PHPLOC;
 
+use function dirname;
+
 class Collector
 {
     private $counts = [];
@@ -21,6 +23,8 @@ class Collector
 
     private $currentMethodLines = 0;
 
+    private $currentNumberOfMethods = 0;
+
     public function getPublisher()
     {
         return new Publisher($this->counts);
@@ -29,7 +33,7 @@ class Collector
     public function addFile($filename): void
     {
         $this->increment('files');
-        $this->addUnique('directories', \dirname($filename));
+        $this->addUnique('directories', dirname($filename));
     }
 
     public function incrementLines($number): void
@@ -55,6 +59,12 @@ class Collector
         }
         $this->currentClassComplexity = 0;
         $this->currentClassLines      = 0;
+        $this->currentNumberOfMethods = 0;
+    }
+
+    public function currentClassStop(): void
+    {
+        $this->addToArray('methods per class', $this->currentNumberOfMethods);
     }
 
     public function currentClassIncrementComplexity(): void
@@ -73,6 +83,11 @@ class Collector
         $this->currentMethodLines      = 0;
     }
 
+    public function currentClassIncrementMethods(): void
+    {
+        $this->currentNumberOfMethods++;
+    }
+
     public function currentMethodIncrementComplexity(): void
     {
         $this->currentMethodComplexity++;
@@ -160,9 +175,14 @@ class Collector
         $this->increment('abstract classes');
     }
 
-    public function incrementConcreteClasses(): void
+    public function incrementNonFinalClasses(): void
     {
-        $this->increment('concrete classes');
+        $this->increment('non-final classes');
+    }
+
+    public function incrementFinalClasses(): void
+    {
+        $this->increment('final classes');
     }
 
     public function incrementNonStaticMethods(): void
@@ -180,9 +200,14 @@ class Collector
         $this->increment('public methods');
     }
 
-    public function incrementNonPublicMethods(): void
+    public function incrementProtectedMethods(): void
     {
-        $this->increment('non-public methods');
+        $this->increment('protected methods');
+    }
+
+    public function incrementPrivateMethods(): void
+    {
+        $this->increment('private methods');
     }
 
     public function incrementNamedFunctions(): void
@@ -200,9 +225,14 @@ class Collector
         $this->increment('global constants');
     }
 
-    public function incrementClassConstants(): void
+    public function incrementPublicClassConstants(): void
     {
-        $this->increment('class constants');
+        $this->increment('public class constants');
+    }
+
+    public function incrementNonPublicClassConstants(): void
+    {
+        $this->increment('non-public class constants');
     }
 
     public function incrementTestClasses(): void
