@@ -1,4 +1,4 @@
-<?php
+<?php declare(strict_types=1);
 /*
  * This file is part of PHPLOC.
  *
@@ -9,6 +9,8 @@
  */
 namespace SebastianBergmann\PHPLOC;
 
-interface Exception
+use Throwable;
+
+interface Exception extends Throwable
 {
 }
