@@ -1,4 +1,4 @@
-<?php
+<?php declare(strict_types=1);
 /*
  * This file is part of PHPLOC.
  *
@@ -9,31 +9,28 @@
  */
 namespace SebastianBergmann\PHPLOC\Log;
 
-/**
- * An XML ResultPrinter for the TextUI.
- */
-class Xml
+use function file_put_contents;
+use DOMDocument;
+
+final class Xml
 {
-    /**
-     * Prints a result set.
-     *
-     * @param string $filename
-     */
-    public function printResult($filename, array $count): void
+    /** @noinspection UnusedFunctionResultInspection */
+    public function printResult(string $filename, array $count): void
     {
-        $document               = new \DOMDocument('1.0', 'UTF-8');
+        $document               = new DOMDocument('1.0', 'UTF-8');
         $document->formatOutput = true;
 
         $root = $document->createElement('phploc');
+
         $document->appendChild($root);
 
         if ($count['directories'] > 0) {
             $root->appendChild(
-                $document->createElement('directories', $count['directories'])
+                $document->createElement('directories', (string) $count['directories'])
             );
 
             $root->appendChild(
-                $document->createElement('files', $count['files'])
+                $document->createElement('files', (string) $count['files'])
             );
         }
 
@@ -41,10 +38,10 @@ class Xml
 
         foreach ($count as $k => $v) {
             $root->appendChild(
-                $document->createElement($k, $v)
+                $document->createElement($k, (string) $v)
             );
         }
 
-        \file_put_contents($filename, $document->saveXML());
+        file_put_contents($filename, $document->saveXML());
     }
 }
