@@ -1,6 +1,6 @@
 <?php
 
-class BaseTest extends PHPUnit_Framework_TestCase {
+class BaseTest extends PHPUnit\Framework\TestCase {
 }
 
 class AIndirectExtendTest extends BaseTest {
